@@ -45,6 +45,56 @@ pub mod arbitrary_precision {
     }
 }
 
+/// Serialize/deserialize optional Decimals as arbitrary precision numbers in JSON using the `arbitrary_precision` feature within `serde_json`.
+///
+/// ```
+/// # use serde::{Serialize, Deserialize};
+/// # use rust_decimal::Decimal;
+/// # use std::str::FromStr;
+///
+/// #[derive(Serialize, Deserialize)]
+/// pub struct ArbitraryExample {
+///     #[serde(with = "rust_decimal::serde::arbitrary_precision_option")]
+///     value: Option<Decimal>,
+/// }
+///
+/// let value = ArbitraryExample { value: Some(Decimal::from_str("123.400").unwrap()) };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":123.400}"#
+/// );
+///
+/// let value = ArbitraryExample { value: None };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":null}"#
+/// );
+/// ```
+#[cfg(feature = "serde-with-arbitrary-precision")]
+pub mod arbitrary_precision_option {
+    use super::*;
+    use serde::Serialize;
+
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        deserializer.deserialize_option(OptionDecimalVisitor)
+    }
+
+    pub fn serialize<S>(value: &Option<Decimal>, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        match *value {
+            Some(ref decimal) => serde_json::Number::from_str(&decimal.to_string())
+                .map_err(serde::ser::Error::custom)?
+                .serialize(serializer),
+            None => serializer.serialize_none(),
+        }
+    }
+}
+
 /// Serialize/deserialize Decimals as floats.
 ///
 /// ```
@@ -85,6 +135,57 @@ pub mod float {
     }
 }
 
+/// Serialize/deserialize optional Decimals as floats.
+///
+/// ```
+/// # use serde::{Serialize, Deserialize};
+/// # use rust_decimal::Decimal;
+/// # use std::str::FromStr;
+///
+/// #[derive(Serialize, Deserialize)]
+/// pub struct FloatExample {
+///     #[serde(with = "rust_decimal::serde::float_option")]
+///     value: Option<Decimal>,
+/// }
+///
+/// let value = FloatExample { value: Some(Decimal::from_str("123.400").unwrap()) };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":123.4}"#
+/// );
+///
+/// let value = FloatExample { value: None };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":null}"#
+/// );
+/// ```
+#[cfg(feature = "serde-with-float")]
+pub mod float_option {
+    use super::*;
+    use serde::Serialize;
+
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        deserializer.deserialize_option(OptionDecimalVisitor)
+    }
+
+    pub fn serialize<S>(value: &Option<Decimal>, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        match *value {
+            Some(ref decimal) => {
+                use num_traits::ToPrimitive;
+                decimal.to_f64().unwrap().serialize(serializer)
+            }
+            None => serializer.serialize_none(),
+        }
+    }
+}
+
 /// Serialize/deserialize Decimals as strings. This is particularly useful when using binary encoding formats.
 ///
 /// ```
@@ -103,6 +204,7 @@ pub mod float {
 ///     &serde_json::to_string(&value).unwrap(),
 ///     r#"{"value":"123.400"}"#
 /// );
+///
 /// ```
 #[cfg(feature = "serde-with-str")]
 pub mod str {
@@ -124,6 +226,56 @@ pub mod str {
     }
 }
 
+/// Serialize/deserialize optional Decimals as strings. This is particularly useful when using binary encoding formats.
+///
+/// ```
+/// # use serde::{Serialize, Deserialize};
+/// # use rust_decimal::Decimal;
+/// # use std::str::FromStr;
+///
+/// #[derive(Serialize, Deserialize)]
+/// pub struct StringExample {
+///     #[serde(with = "rust_decimal::serde::str_option")]
+///     value: Option<Decimal>,
+/// }
+///
+/// let value = StringExample { value: Some(Decimal::from_str("123.400").unwrap()) };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":"123.400"}"#
+/// );
+///
+/// let value = StringExample { value: None };
+/// assert_eq!(
+///     &serde_json::to_string(&value).unwrap(),
+///     r#"{"value":null}"#
+/// );
+/// ```
+#[cfg(feature = "serde-with-str")]
+pub mod str_option {
+    use super::*;
+
+    pub fn deserialize<'de, D>(deserializer: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        deserializer.deserialize_option(OptionDecimalStrVisitor)
+    }
+
+    pub fn serialize<S>(value: &Option<Decimal>, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        match *value {
+            Some(ref decimal) => {
+                let decimal = crate::str::to_str_internal(decimal, true, None);
+                serializer.serialize_some::<str>(decimal.0.as_ref())
+            }
+            None => serializer.serialize_none(),
+        }
+    }
+}
+
 #[cfg(not(feature = "serde-str"))]
 impl<'de> serde::Deserialize<'de> for Decimal {
     fn deserialize<D>(deserializer: D) -> Result<Decimal, D::Error>
@@ -218,6 +370,81 @@ impl<'de> serde::de::Visitor<'de> for DecimalVisitor {
     }
 }
 
+struct OptionDecimalVisitor;
+
+impl<'de> serde::de::Visitor<'de> for OptionDecimalVisitor {
+    type Value = Option<Decimal>;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str("a Decimal type representing a fixed-point number")
+    }
+
+    fn visit_none<E>(self) -> Result<Option<Decimal>, E>
+    where
+        E: serde::de::Error,
+    {
+        Ok(None)
+    }
+
+    #[cfg(all(feature = "serde-str", feature = "serde-float"))]
+    fn visit_some<D>(self, d: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        // We've got multiple types that we may see so we need to use any
+        d.deserialize_any(DecimalVisitor).map(Some)
+    }
+
+    #[cfg(not(all(feature = "serde-str", feature = "serde-float")))]
+    fn visit_some<D>(self, d: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        <Decimal as serde::Deserialize>::deserialize(d).map(Some)
+    }
+}
+
+#[cfg(feature = "serde-with-str")]
+struct OptionDecimalStrVisitor;
+
+#[cfg(feature = "serde-with-str")]
+impl<'de> serde::de::Visitor<'de> for OptionDecimalStrVisitor {
+    type Value = Option<Decimal>;
+
+    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        formatter.write_str("a Decimal type representing a fixed-point number")
+    }
+
+    fn visit_none<E>(self) -> Result<Option<Decimal>, E>
+    where
+        E: serde::de::Error,
+    {
+        Ok(None)
+    }
+
+    fn visit_some<D>(self, d: D) -> Result<Option<Decimal>, D::Error>
+    where
+        D: serde::de::Deserializer<'de>,
+    {
+        d.deserialize_str(Self)
+    }
+
+    fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
+    where
+        E: serde::de::Error,
+    {
+        match v.is_empty() {
+            true => Ok(None),
+            false => {
+                let d = Decimal::from_str(v)
+                    .or_else(|_| Decimal::from_scientific(v))
+                    .map_err(serde::de::Error::custom)?;
+                Ok(Some(d))
+            }
+        }
+    }
+}
+
 #[cfg(feature = "serde-with-arbitrary-precision")]
 struct DecimalKey;
 
@@ -343,8 +570,7 @@ mod test {
         ];
         for &(serialized, value) in data.iter() {
             let result = serde_json::from_str(serialized);
-            assert_eq!(
-                true,
+            assert!(
                 result.is_ok(),
                 "expected successful deserialization for {}. Error: {:?}",
                 serialized,
@@ -356,7 +582,7 @@ mod test {
                 record.amount.to_string(),
                 "expected: {}, actual: {}",
                 value,
-                record.amount.to_string()
+                record.amount
             );
         }
     }
@@ -432,7 +658,7 @@ mod test {
 
     #[test]
     #[cfg(all(feature = "serde-str", not(feature = "serde-float")))]
-    fn bincode_serialization() {
+    fn bincode_serialization_not_float() {
         use bincode::{deserialize, serialize};
 
         let data = [
@@ -456,7 +682,7 @@ mod test {
 
     #[test]
     #[cfg(all(feature = "serde-str", feature = "serde-float"))]
-    fn bincode_serialization() {
+    fn bincode_serialization_serde_float() {
         use bincode::{deserialize, serialize};
 
         let data = [
@@ -558,7 +784,7 @@ mod test {
         use bincode::{deserialize, serialize};
 
         #[derive(Serialize, Deserialize)]
-        pub struct BincodeExample {
+        struct BincodeExample {
             #[serde(with = "crate::serde::str")]
             value: Decimal,
         }
@@ -566,6 +792,7 @@ mod test {
         let data = [
             ("0", "0"),
             ("0.00", "0.00"),
+            ("1.234", "1.234"),
             ("3.14159", "3.14159"),
             ("-3.14159", "-3.14159"),
             ("1234567890123.4567890", "1234567890123.4567890"),
@@ -581,4 +808,112 @@ mod test {
             assert_eq!(expected, decoded.value);
         }
     }
+
+    #[test]
+    #[cfg(feature = "serde-with-str")]
+    fn with_str_bincode_optional() {
+        use bincode::{deserialize, serialize};
+
+        #[derive(Serialize, Deserialize)]
+        struct BincodeExample {
+            #[serde(with = "crate::serde::str_option")]
+            value: Option<Decimal>,
+        }
+
+        // Some(value)
+        let value = Some(Decimal::new(1234, 3));
+        let input = BincodeExample { value };
+        let encoded = serialize(&input).unwrap();
+        let decoded: BincodeExample = deserialize(&encoded[..]).unwrap();
+        assert_eq!(value, decoded.value, "Some(value)");
+
+        // None
+        let input = BincodeExample { value: None };
+        let encoded = serialize(&input).unwrap();
+        let decoded: BincodeExample = deserialize(&encoded[..]).unwrap();
+        assert_eq!(None, decoded.value, "None");
+    }
+
+    #[test]
+    #[cfg(feature = "serde-with-str")]
+    fn with_str_optional() {
+        #[derive(Serialize, Deserialize)]
+        pub struct StringExample {
+            #[serde(with = "crate::serde::str_option")]
+            value: Option<Decimal>,
+        }
+
+        let original = StringExample {
+            value: Some(Decimal::from_str("123.400").unwrap()),
+        };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":"123.400"}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":"123.400"}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_some());
+        assert_eq!(deserialized.value.unwrap().unpack(), original.value.unwrap().unpack());
+
+        // Null tests
+        let original = StringExample { value: None };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":null}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":null}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_none());
+
+        // Empty string deserialization tests
+        let original = StringExample { value: None };
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":""}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_none());
+    }
+
+    #[test]
+    #[cfg(feature = "serde-with-float")]
+    fn with_float_optional() {
+        #[derive(Serialize, Deserialize)]
+        pub struct StringExample {
+            #[serde(with = "crate::serde::float_option")]
+            value: Option<Decimal>,
+        }
+
+        let original = StringExample {
+            value: Some(Decimal::from_str("123.400").unwrap()),
+        };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":123.4}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":123.4}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_some()); // Scale is different!
+
+        // Null tests
+        let original = StringExample { value: None };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":null}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":null}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_none());
+    }
+
+    #[test]
+    #[cfg(feature = "serde-with-arbitrary-precision")]
+    fn with_arbitrary_precision_optional() {
+        #[derive(Serialize, Deserialize)]
+        pub struct StringExample {
+            #[serde(with = "crate::serde::arbitrary_precision_option")]
+            value: Option<Decimal>,
+        }
+
+        let original = StringExample {
+            value: Some(Decimal::from_str("123.400").unwrap()),
+        };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":123.400}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":123.400}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_some());
+        assert_eq!(deserialized.value.unwrap().unpack(), original.value.unwrap().unpack());
+
+        // Null tests
+        let original = StringExample { value: None };
+        assert_eq!(&serde_json::to_string(&original).unwrap(), r#"{"value":null}"#);
+        let deserialized: StringExample = serde_json::from_str(r#"{"value":null}"#).unwrap();
+        assert_eq!(deserialized.value, original.value);
+        assert!(deserialized.value.is_none());
+    }
 }
